package com.coherentlogic.fred.client.core.domain;

import java.io.Serializable;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.Table;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

import static com.coherentlogic.fred.client.core.util.Constants.DEFAULT_OBJECT;

/**
 * A class which all domain classes inherit from and that contains
 * implementations of the hashCode, equals, and toString methods.
 *
 * @author <a href="support@coherentlogic.com">Support</a>
 */
@Entity()
@Table(name=DEFAULT_OBJECT)
@Inheritance(strategy=InheritanceType.TABLE_PER_CLASS)
public class DefaultObject implements Serializable {

    private static final long serialVersionUID = -1914917862686826505L;

    private Long primaryKey = null;

    public DefaultObject () {
    }

    @Override
    public int hashCode () {
        return HashCodeBuilder.reflectionHashCode(this);
    }

    @Override
    public boolean equals (Object obj) {
        return EqualsBuilder.reflectionEquals(this, obj);
    }

    @Id
    @GeneratedValue(strategy=GenerationType.TABLE)
    public Long getPrimaryKey() {
        return primaryKey;
    }

    public void setPrimaryKey(Long primaryKey) {
        this.primaryKey = primaryKey;
    }

    @Override
    public String toString() {
        return ToStringBuilder.reflectionToString(this);
    }
}
