package com.coherentlogic.fred.client.core.domain;

import static com.coherentlogic.fred.client.core.util.Constants.COUNT;
import static com.coherentlogic.fred.client.core.util.Constants.COUNT_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.LIMIT;
import static com.coherentlogic.fred.client.core.util.Constants.LIMIT_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.OFFSET;
import static com.coherentlogic.fred.client.core.util.Constants.OFFSET_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.ORDER_BY;
import static com.coherentlogic.fred.client.core.util.Constants.ORDER_BY_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_END;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_START;
import static com.coherentlogic.fred.client.core.util.Constants.RELEASE_DATES;
import static com.coherentlogic.fred.client.core.util.Constants.SORT_ORDER;

import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;

/**
 * A class which represents the release dates for all releases of economic data.
 *
 * @see <a href="http://api.stlouisfed.org/docs/fred/releases_dates.html">
 * releases_dates</a>
 * @see <a href="http://api.stlouisfed.org/docs/fred/release_dates.html">
 * release_dates</a>
 *
 * @author <a href="support@coherentlogic.com">Support</a>
 */
@Entity
@Table(name=RELEASE_DATES)
@XStreamAlias(RELEASE_DATES)
public class ReleaseDates extends IdentityObject
    implements RealtimeBoundBean, PaginationBean, OrderByBean, SortOrderBean {

    private static final long serialVersionUID = -8105863815832802092L;

    @XStreamImplicit
    private List<ReleaseDate> releaseDateList = null;

    @XStreamAlias(REALTIME_START)
    @XStreamAsAttribute
    private Date realtimeStartDate = null;

    @XStreamAlias(REALTIME_END)
    @XStreamAsAttribute
    private Date realtimeEndDate = null;

    @XStreamAlias(SORT_ORDER)
    @XStreamAsAttribute
    private SortOrder sortOrder = SortOrder.asc;

    @XStreamAlias(ORDER_BY)
    @XStreamAsAttribute
    private OrderBy orderBy = null;

    @XStreamAlias(LIMIT)
    @XStreamAsAttribute
    private long limit = 1000;

    @XStreamAlias(OFFSET)
    @XStreamAsAttribute
    private int offset = 0;

    @XStreamAlias(COUNT)
    @XStreamAsAttribute
    private int count = 0;

    @Override
    public void setSortOrder(SortOrder sortOrder) {
        this.sortOrder = sortOrder;
    }

    @Override
    public SortOrder getSortOrder() {
        return sortOrder;
    }

    @Column(name=ORDER_BY_VALUE)
    @Override
    public OrderBy getOrderBy() {
        return orderBy;
    }

    @Override
    public void setOrderBy(OrderBy orderBy) {
        this.orderBy = orderBy;
    }

    @Override
    public void setLimit(long limit) {
        this.limit = limit;
    }

    @Override
    public void setOffset(int offset) {
        this.offset = offset;
    }

    /**
     * In the H2 database the limit is a reserved keyword so we are renaming
     * this, otherwise we'll see an exception along the lines of "Table
     * 'DEFAULTOBJECT' not found; SQL statement".
     */
    @Column(name=LIMIT_VALUE)
    @Override
    public long getLimit() {
        return limit;
    }

    @Column(name=OFFSET_VALUE)
    @Override
    public int getOffset() {
        return offset;
    }

    @OneToMany(cascade=CascadeType.ALL)
    public List<ReleaseDate> getReleaseDateList() {
        return releaseDateList;
    }

    public void setReleaseDateList(List<ReleaseDate> releaseDateList) {
        this.releaseDateList = releaseDateList;
    }

    @Column(name=COUNT_VALUE)
    public int getCount() {
        return count;
    }

    public void setCount(int count) {
        this.count = count;
    }

    @Override
    public void setRealtimeStart(Date realtimeStartDate) {
        this.realtimeStartDate = realtimeStartDate;
    }

    @Override
    public void setRealtimeEnd(Date realtimeEndDate) {
        this.realtimeEndDate = realtimeEndDate;
    }

    @Override
    public Date getRealtimeStart() {
        return realtimeStartDate;
    }

    @Override
    public Date getRealtimeEnd() {
        return realtimeEndDate;
    }
}
