package com.coherentlogic.fred.client.core.domain;

import static com.coherentlogic.fred.client.core.util.Constants.COUNT;
import static com.coherentlogic.fred.client.core.util.Constants.FILE_TYPE;
import static com.coherentlogic.fred.client.core.util.Constants.FILTER_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.FILTER_VARIABLE;
import static com.coherentlogic.fred.client.core.util.Constants.LIMIT;
import static com.coherentlogic.fred.client.core.util.Constants.LIMIT_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.OFFSET;
import static com.coherentlogic.fred.client.core.util.Constants.OFFSET_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.ORDER_BY;
import static com.coherentlogic.fred.client.core.util.Constants.ORDER_BY_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.OUTPUT_TYPE;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_END;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_START;
import static com.coherentlogic.fred.client.core.util.Constants.SERIESS;
import static com.coherentlogic.fred.client.core.util.Constants.SORT_ORDER;
import static com.coherentlogic.fred.client.core.util.Constants.UNITS;

import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;

/**
 * A class which represents an economic data series.
 *
 * @see <a href="http://api.stlouisfed.org/docs/fred/category_series.html">
 * category_series</a>
 * @see <a href="http://api.stlouisfed.org/docs/fred/release_series.html">
 * release_series</a>
 * @see <a href="http://api.stlouisfed.org/docs/fred/series.html">series</a>
 *
 * @author <a href="support@coherentlogic.com">Support</a>
 */
@Entity
@Table(name=SERIESS)
@XStreamAlias(SERIESS)
public class Seriess extends DefaultObject
    implements RealtimeBoundBean,
        FilterBean, PaginationBean, OrderByBean, SortOrderBean {

    private static final long serialVersionUID = -2127629128549568067L;

    @XStreamAlias(REALTIME_START)
    @XStreamAsAttribute
    private Date realtimeStart = null;

    @XStreamAlias(REALTIME_END)
    @XStreamAsAttribute
    private Date realtimeEnd = null;

    @XStreamAlias(SORT_ORDER)
    @XStreamAsAttribute
    private SortOrder sortOrder = SortOrder.asc;

    @XStreamAlias(ORDER_BY)
    @XStreamAsAttribute
    private OrderBy orderBy = OrderBy.seriesId;

    @XStreamAlias(LIMIT)
    @XStreamAsAttribute
    private long limit = 1000;

    @XStreamAlias(OFFSET)
    @XStreamAsAttribute
    private int offset = 0;

    @XStreamAlias(UNITS)
    @XStreamAsAttribute
    private Unit units = null;

    @XStreamAlias(OUTPUT_TYPE)
    @XStreamAsAttribute
    private OutputType outputType = null;

    @XStreamAlias(FILE_TYPE)
    @XStreamAsAttribute
    private FileType fileType = null;

    @XStreamAlias(COUNT)
    @XStreamAsAttribute
    private int count = 0;

    @XStreamAlias(FILTER_VARIABLE)
    @XStreamAsAttribute
    private FilterVariable filterVariable = null;

    @XStreamAlias(FILTER_VALUE)
    @XStreamAsAttribute
    private FilterValue filterValue = null;

    @XStreamImplicit
    private List<Series> seriesList = null;

    @OneToMany(cascade=CascadeType.ALL, fetch=FetchType.LAZY)
    public List<Series> getSeriesList () {
        return seriesList;
    }

    public void setSeriesList (List<Series> seriesList) {
        this.seriesList = seriesList;
    }

    @Override
    public void setRealtimeStart(Date realtimeStart) {
        this.realtimeStart = realtimeStart;
    }

    @Override
    public void setRealtimeEnd(Date realtimeEnd) {
        this.realtimeEnd = realtimeEnd;
    }

    @Override
    public Date getRealtimeStart() {
        return realtimeStart;
    }

    @Override
    public Date getRealtimeEnd() {
        return realtimeEnd;
    }

    @Override
    public void setSortOrder(SortOrder sortOrder) {
        this.sortOrder = sortOrder;
    }

    @Override
    public SortOrder getSortOrder() {
        return sortOrder;
    }

    @Column(name=ORDER_BY_VALUE)
    @Override
    public OrderBy getOrderBy() {
        return orderBy;
    }

    @Override
    public void setOrderBy(OrderBy orderBy) {
        this.orderBy = orderBy;
    }

    @Override
    public void setLimit(long limit) {
        this.limit = limit;
    }

    @Override
    public void setOffset(int offset) {
        this.offset = offset;
    }

    /**
     * In the H2 database the limit is a reserved keyword so we are renaming
     * this, otherwise we'll see an exception along the lines of "Table
     * 'DEFAULTOBJECT' not found; SQL statement".
     */
    @Column(name=LIMIT_VALUE)
    @Override
    public long getLimit() {
        return limit;
    }

    @Column(name=OFFSET_VALUE)
    @Override
    public int getOffset() {
        return offset;
    }

    @Override
    public FilterVariable getFilterVariable() {
        return filterVariable;
    }

    @Override
    public void setFilterVariable(FilterVariable filterVariable) {
        this.filterVariable = filterVariable;
    }

    @Override
    public FilterValue getFilterValue() {
        return filterValue;
    }

    @Override
    public void setFilterValue(FilterValue filterValue) {
        this.filterValue = filterValue;
    }

    public Unit getUnits() {
        return units;
    }

    public void setUnits(Unit units) {
        this.units = units;
    }

    public OutputType getOutputType() {
        return outputType;
    }

    public void setOutputType(OutputType outputType) {
        this.outputType = outputType;
    }

    public FileType getFileType() {
        return fileType;
    }

    public void setFileType(FileType fileType) {
        this.fileType = fileType;
    }

    public int getCount() {
        return count;
    }

    public void setCount(int count) {
        this.count = count;
    }
}
