package com.coherentlogic.fred.client.core.domain;

import static com.coherentlogic.fred.client.core.util.Constants.LINK;
import static com.coherentlogic.fred.client.core.util.Constants.NAME;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_END;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_START;
import static com.coherentlogic.fred.client.core.util.Constants.SOURCE;

import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.Table;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;

/**
 * A class which represents a source of economic data.
 *
 * @see <a href="http://api.stlouisfed.org/docs/fred/release_sources.html">
 * release_sources</a>
 * @see <a href="http://api.stlouisfed.org/docs/fred/sources.html">
 * sources</a>
 * @see <a href="http://api.stlouisfed.org/docs/fred/source.html">
 * source</a>
 *
 * @author <a href="support@coherentlogic.com">Support</a>
 */
@Entity
@Table(name=SOURCE)
@XStreamAlias(SOURCE)
public class Source extends IdentityObject implements RealtimeBoundBean {

    private static final long serialVersionUID = -3054323864047172358L;

    /**
     * For example "2008-07-29".
     */
    @XStreamAlias(REALTIME_START)
    @XStreamAsAttribute
    private Date realtimeStart = null;

    /**
     * For example "2008-07-29".
     */
    @XStreamAlias(REALTIME_END)
    @XStreamAsAttribute
    private Date realtimeEnd = null;

    /**
     * For example "Real Gross National Product".
     */
    @XStreamAlias(NAME)
    @XStreamAsAttribute
    private String name = null;

    @XStreamAlias(LINK)
    @XStreamAsAttribute
    private String link = null;

    @Override
    public Date getRealtimeStart () {
        return realtimeStart;
    }

    @Override
    public void setRealtimeStart (Date realtimeStart) {
        this.realtimeStart = realtimeStart;
    }

    @Override
    public Date getRealtimeEnd () {
        return realtimeEnd;
    }

    @Override
    public void setRealtimeEnd (Date realtimeEnd) {
        this.realtimeEnd = realtimeEnd;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getLink() {
        return link;
    }

    public void setLink(String link) {
        this.link = link;
    }
}
