package com.coherentlogic.fred.client.core.domain;

import static com.coherentlogic.fred.client.core.util.Constants.COUNT;
import static com.coherentlogic.fred.client.core.util.Constants.COUNT_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.LIMIT;
import static com.coherentlogic.fred.client.core.util.Constants.LIMIT_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.OFFSET;
import static com.coherentlogic.fred.client.core.util.Constants.OFFSET_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.ORDER_BY;
import static com.coherentlogic.fred.client.core.util.Constants.ORDER_BY_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_END;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_START;
import static com.coherentlogic.fred.client.core.util.Constants.SORT_ORDER;
import static com.coherentlogic.fred.client.core.util.Constants.VINTAGE_DATES;

import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;

/**
 * A class which represents the dates in history when a series' data values were
 * revised or new data values were released.
 *
 * @see <a href="http://api.stlouisfed.org/docs/fred/series_vintagedates.html">
 * series_vintagedates</a>
 *
 * @author <a href="support@coherentlogic.com">Support</a>
 */
@Entity
@Table(name=VINTAGE_DATES)
@XStreamAlias(VINTAGE_DATES)
public class VintageDates extends DefaultObject
    implements RealtimeBoundBean, PaginationBean, SortOrderBean {

    private static final long serialVersionUID = -7156068168926360344L;

    @XStreamAlias(REALTIME_START)
    @XStreamAsAttribute
    private Date realtimeStart = null;

    @XStreamAlias(REALTIME_END)
    @XStreamAsAttribute
    private Date realtimeEnd = null;

    @XStreamAlias(SORT_ORDER)
    @XStreamAsAttribute
    private SortOrder sortOrder = SortOrder.asc;

    /**
     * @todo Should this be the enum OrderBy?
     */
    @XStreamAlias(ORDER_BY)
    @XStreamAsAttribute
    private String orderBy = null;

    @XStreamAlias(LIMIT)
    @XStreamAsAttribute
    private long limit = 1000;

    @XStreamAlias(OFFSET)
    @XStreamAsAttribute
    private int offset = 0;

    @XStreamAlias(COUNT)
    @XStreamAsAttribute
    private int count = 0;

    @XStreamImplicit
    private List<VintageDate> vintageDateList = null;

    @OneToMany(cascade=CascadeType.ALL)
    public List<VintageDate> getVintageDateList() {
        return vintageDateList;
    }

    public void setVintageDateList(List<VintageDate> vintageDateList) {
        this.vintageDateList = vintageDateList;
    }

    @Override
    public void setSortOrder(SortOrder sortOrder) {
        this.sortOrder = sortOrder;
    }

    @Override
    public SortOrder getSortOrder() {
        return sortOrder;
    }

    @Column(name=ORDER_BY_VALUE)
    public String getOrderBy() {
        return orderBy;
    }

    public void setOrderBy(String orderBy) {
        this.orderBy = orderBy;
    }

    @Override
    public void setLimit(long limit) {
        this.limit = limit;
    }

    @Override
    public void setOffset(int offset) {
        this.offset = offset;
    }

    /**
     * In the H2 database the limit is a reserved keyword so we are renaming
     * this, otherwise we'll see an exception along the lines of "Table
     * 'DEFAULTOBJECT' not found; SQL statement".
     */
    @Column(name=LIMIT_VALUE)
    @Override
    public long getLimit() {
        return limit;
    }

    @Column(name=OFFSET_VALUE)
    @Override
    public int getOffset() {
        return offset;
    }

    @Override
    public void setRealtimeStart(Date realtimeStart) {
        this.realtimeStart = realtimeStart;
    }

    @Override
    public void setRealtimeEnd(Date realtimeEnd) {
        this.realtimeEnd = realtimeEnd;
    }

    @Override
    public Date getRealtimeStart() {
        return realtimeStart;
    }

    @Override
    public Date getRealtimeEnd() {
        return realtimeEnd;
    }

    @Column(name=COUNT_VALUE)
    public int getCount() {
        return count;
    }

    public void setCount(int count) {
        this.count = count;
    }
}
