package com.coherentlogic.fred.client.core.factories;

import org.springframework.web.client.RestTemplate;

import com.coherentlogic.fred.client.core.builders.RequestBuilder;

/**
 * A factory which is used for creating instances of RequestBuilder.
 * <p>
 * Since the RequestBuilder is not thread-safe, it cannot be set as a member
 * variable -- instead, use this factory and call {@link #getInstance()}
 * whenever you need to query the FRED web services.
 * <p>
 * This class can be extended, for example, if you need to always return the
 * same value for some query parameter -- for example, in this class we always
 * return the same API key.
 * <p>
 * @author <a href="support@coherentlogic.com">Support</a>
 */
public class RequestBuilderFactory
    implements Factory<RequestBuilder> {

    private final RestTemplate restTemplate;

    private final String uri;

    private final String apiKey;

    public RequestBuilderFactory (
        RestTemplate restTemplate,
        String uri,
        String apiKey
    ) {
        this.restTemplate = restTemplate;
        this.uri = uri;
        this.apiKey = apiKey;
    }

    @Override
    public RequestBuilder getInstance() {

        RequestBuilder result = new RequestBuilder (restTemplate, uri);

        result.setApiKey(apiKey);

        return result;
    }
}
